# brightsign_source_release
#
# This class exists in order to automatically generate the files required for the BrightSign source release. It does three separate things:
#
# 1. The remote_sources task ensures that the filenames of source
#    tarballs are made available to the release-source script.
#
# 2. The husk_recipe task generates "husk" recipes for packages that
#    the source is not shipped for so that other recipes can depend on
#    them and vice-versa. This ensures that we're releasing the source
#    for and able to build all dependencies.
#
# 3. Creates a new source_release task that the above tasks and other
#    recipes that have more complex source release requirements can
#    hook into.
#
# The aim is for everything that is required to generate the source
# release for a given image to be built by the all_source_releases
# task.

# Licenses for stuff we need to include in our source release.  The
# lmbench-proprietary license is basically GPLv2 as far as we're
# concerned.
REMOTE_SOURCES_INCLUDE_LICENSES = "\
	AGPL-3.0 \
	GPL \
	GPL-1.0 \
	GPL-2+ \
	GPL-2.0+ \
	GPLv1+ \
	GPLv2 GPLv2+ GPL-v2 GPL-2.0 \
	GPLv3 GPLv3+ GPL-v3 GPL-3.0-with-autoconf-exception \
        GPL-2.0-with-GCC-exception \
        GPL-2.0-with-classpath-exception \
	LGPL \
	LGPLv2 LGPLv2+ LGPL-v2 LGPLv2.0+ \
	LGPLv2.1 LGPLv2.1+ LGPL-v2.1 LGPL2.1 LGPL-2.1+\
        LGPLv3 \
	FSF-Unlimited \
	Apache-2.0 \
	Apache-2 \
	Artistic \
	Artistic-1.0 \
        BitstreamVera \
	Boost \
	BSD \
	NewBSD \
	BSD-2-Clause \
	BSD-3-Clause \
        ISC \
	bzip2 \
	Info-ZIP \
	ICU \
	jpeg \
	Libpng \
	MIT \
	MIT-style \
        openssl \
	PD \
	PSF \
	PSFv2 \
	Sleepycat \
	Zlib \
	GPLv3+exception \
        lmbench-proprietary \
"

# We don't actually ship stuff that uses these licenses, but we use
# the code in order to build what we ship. Unfortunately they don't
# all inherit from native.bbclass so we can't use virtclass-native for
# this.
REMOTE_SOURCES_INCLUDE_NATIVE_LICENSES = "\
        EPL-1.0 \
        CUP \
        IBM \
        JLEX \
        CPL-1.0 \
        JACL \
        MPL-2.0 \
"

# Licenses for stuff we explicitly need to exclude from our source release
REMOTE_SOURCES_EXCLUDE_LICENSES = "\
	Proprietary \
	CLOSED \
"

# We normally use GZIP=-1 for speed but for the source release we'd
# like to use real compression. We'd also like the same input to
# always generate the same output so that we can produce multiple
# tarballs of the same source (e.g. Puma and Panther kernels) that
# have identical hashes.
do_source_release_prepend() {
    # The gzip way to produce deterministic output is to say "-n"
    GZIP="-9 -n"
    export GZIP

    # But pigz requires "-n" and "--no-time". It looks at GZIP too
    # though so we needn't repeat the options from there.
    PIGZ="--no-time"
    export PIGZ
}

do_source_release() {
	:
}
do_source_release[nostamp] = "1"

# Ensure that we don't rely on ${S} existing for the task to work.
do_source_release[dirs] = "${WORKDIR}"

EXPORT_FUNCTIONS do_source_release

addtask all_source_releases
brightsign_source_release_do_all_source_releases() {
    :
}

do_all_source_releases[nostamp] = "1"
do_all_source_releases[recrdeptask] += "do_all_source_releases do_source_release"

EXPORT_FUNCTIONS do_all_source_releases

addtask source_release

# Ensure that we don't rely on ${S} existing for the recipe to work.
do_remote_sources[dirs] = "${WORKDIR}"

python brightsign_source_release_do_remote_sources() {
    import re

    # Let recipes override the sources list used for this. In
    # particular directfb needs to do this.
    src_uri = (bb.data.getVar('SOURCE_RELEASE_SRC_URI', d, True) or "").split()
    if len(src_uri) == 0:
        src_uri = (bb.data.getVar('SRC_URI', d, True) or "").split()
        if len(src_uri) == 0:
            return
    
    # Should use generic license stuff when we merge it. This is a bit
    # naughty because we end up combining both AND and OR and ignore
    # brackets.
    licenses = filter(bool, [ l.strip() for l in re.split('[&|()]', bb.data.getVar('LICENSE', d, True)) ])
    
    include_licenses = set(bb.data.getVar('REMOTE_SOURCES_INCLUDE_LICENSES', d, True).split())
    if (d.getVar('PN', True).endswith("-native")):
        include_licenses |= set(bb.data.getVar('REMOTE_SOURCES_INCLUDE_NATIVE_LICENSES', d, True).split())
    exclude_licenses = set(bb.data.getVar('REMOTE_SOURCES_EXCLUDE_LICENSES', d, True).split())
    include_pkg = False
    exclude_pkg = False    
    for lic in licenses:
        if lic in include_licenses:
            include_pkg = True
        elif lic in exclude_licenses:
            exclude_pkg = True
    
    if include_pkg and exclude_pkg:
        bb.fatal("Package ${PN} has conflicting licenses '${LICENSE}'")

    if exclude_pkg:
        bb.note("${PN} license is excluded from remote_sources")
        return
    
    if not include_pkg:
        bb.fatal("Package ${PN} has licenses '${LICENSE}' which are unknown to remote_sources")
        return

    if src_uri == [ "unpublished" ]:
        bb.fatal("${PN} license means we should list remote_sources but the URI is %s" % src_uri)
        return
    
    localdata = bb.data.createCopy(d)
    bb.data.update_data(localdata)
    
    try:
        fetcher = bb.fetch2.Fetch(src_uri, localdata)
        bb.utils.mkdirhier(bb.data.expand("${DEPLOY_DIR}/remote_sources/${MACHINE}", d))
        f = open(bb.data.expand("${DEPLOY_DIR}/remote_sources/${MACHINE}/${PN}", d), 'w')
        dl_dir = bb.data.getVar('DL_DIR', d, True)
        for l in fetcher.localpaths():
            if l.startswith(dl_dir):
                l = l[len(dl_dir):]
                if l.startswith("/"):
                    l = l[1:]
                # Convert git repository to its tarball. This is hacky!
                if l.startswith("git2/"):
                    l = l.replace("/", "_")
                    l += ".tar.gz"
                f.write("%s\n" % l)
        f.close
    except bb.fetch2.BBFetchException, e:
        raise bb.build.FuncFailed(e)
}

# If we want to be sure that the information is up to date we have to
# generate it every time. :(
do_remote_sources[nostamp] = "1"
    
addtask remote_sources before do_source_release after do_fetch

# Ensure that we don't rely on ${S} existing for the recipe to work.
do_clean_remote_sources[dirs] = "${WORKDIR}"

brightsign_source_release_do_clean_remote_sources() {
    rm -f "${DEPLOY_DIR}/remote-sources/${MACHINE}/${PN}"
}

addtask clean_remote_sources before do_clean

EXPORT_FUNCTIONS do_remote_sources do_clean_remote_sources

# Licenses for stuff we explicitly need to exclude from our source
# releases but need to include husk recipes for to keep the dependency
# chain going. If we're not shipping the source then we must need a
# husk recipe.
HUSK_RECIPES_INCLUDE_LICENSES = "${REMOTE_SOURCES_EXCLUDE_LICENSES}"

# Ensure that we don't rely on ${S} existing for the recipe to work.
do_husk_recipe[dirs] = "${WORKDIR}"

# This recipe generates husky recipes for all closed source recipes so
# that their dependencies can be built
python brightsign_source_release_do_husk_recipe() {
    import re, string
    
    # Should use generic license stuff when we merge it. This is a bit
    # naughty because we end up combining both AND and OR and ignore
    # brackets.
    licenses = filter(bool, [ l.strip() for l in re.split('[&|()]', bb.data.getVar('LICENSE', d, True)) ])
    
    include_licenses = set(bb.data.getVar('HUSK_RECIPES_INCLUDE_LICENSES', d, True).split())
    include_pkg = False
    for lic in licenses:
        if lic in include_licenses:
            include_pkg = True
    
    if not include_pkg:
        bb.note("${PN} license %s is excluded from husk_recipes" % string.join(licenses))
        return

    pn = bb.data.getVar('PN', d, True)
    depends = bb.data.getVar('DEPENDS', d, True).split()
    rdepends = bb.data.getVar('RDEPENDS', d, True).split()
    provides = bb.data.getVar('RDEPENDS', d, True).split()
    packages = bb.data.getVar('PACKAGES', d, True).split()

    # Some recipes have dependencies only in the source release
    extra_depends =  bb.data.getVar('SOURCE_RELEASE_EXTRA_DEPENDS', d, True)
    if extra_depends:
       depends = depends + extra_depends.split()

    # Let's work out what the dependencies are for this recipe
#    bb.warn("RECIPE: %s" % pn)
#    bb.warn("DEPENDS: %s" % depends)
#    bb.warn("RDEPENDS: %s" % rdepends)
#    bb.warn("PROVIDES: %s" % provides)
#    bb.warn("PACKAGES: %s" % packages)

    try:
        os.makedirs(bb.data.expand("${DEPLOY_DIR}/oe-generated/meta-bs-husk/recipes-bs/${PN}", d))
    except os.error:
        pass

    f = open(bb.data.expand("${DEPLOY_DIR}/oe-generated/meta-bs-husk/recipes-bs/${PN}/${PN}-${MACHINE}.bb", d), 'w')
    f.write("# %s\n" % pn)
    f.write("#\n")
    f.write("# This is an automatically generated husk recipe. It exists only to allow\n")
    f.write("# things to depend on it and to depend on other things. It is used to ensure\n")
    f.write("# that the compilation of everything is tested by BrightSign when generating\n")
    f.write("# the source release.\n")
    f.write("\n")
    f.write("LICENSE = \"CLOSED\"\n")
    f.write("COMPATIBLE_MACHINE = \"%s\"\n" % bb.data.getVar('MACHINE', d, True))
    f.write("PACKAGE_ARCH = \"$" + "{MACHINE_ARCH}\"\n")
    if len(depends) > 0:
        f.write("DEPENDS = \"%s\"\n" % string.join(depends))

    if len(rdepends) > 0:
        f.write("RDEPENDS = \"%s\"\n" % string.join(rdepends))

    if len(provides) > 0:
        f.write("PROVIDES = \"%s\"\n" % string.join(provides))
    else:
        f.write("# Implicit provides\nPROVIDES = \"%s\"\n" % pn)

    if len(packages) > 0:
        f.write("PACKAGES = \"%s\"\n" % string.join(packages))

    # We need to generate the packages even if they are empty so they can be installed
    for package in packages:
        f.write("ALLOW_EMPTY_" + package + " = \"1\"\n")

    f.close
}

# If we want to be sure that the information is up to date we have to
# generate it every time. :(
do_husk_recipe[nostamp] = "1"
    
addtask husk_recipe before do_source_release

# Ensure that we don't rely on ${S} existing for the recipe to work.
do_clean_husk_recipe[dirs] = "${WORKDIR}"

brightsign_source_release_do_clean_husk_recipe() {
    rm -f "${DEPLOY_DIR}/oe-generated/meta-bs-husk/recipes-bs/${PN}/${PN}.bb"
}
do_clean_husk_recipe[nostamp] = "1"

addtask clean_husk_recipe before do_clean

EXPORT_FUNCTIONS do_husk_recipe do_clean_husk_recipe

